GLOBALS_SECTION
  #include <fvar.hpp>
  #include <math.h>

  _CONST double log2pi = log(2.0*M_PI);

  void dmdm_bksolve(void);

  dvar_matrix bksolve(_CONST dvar_matrix& L, _CONST dvar_matrix& b)
  {
    RETURN_ARRAYS_INCREMENT();      
    int i, k, r, R, c, C;
    r=b.rowmin();
    R=b.rowmax();
    c=b.colmin();
    C=b.colmax();
  
    dvector sumVec(c,C);
    dmatrix xVal(r,R,c,C);
    dmatrix LVal=value(L);
    dmatrix bVal=value(b);
  
    for(i=r; i<=R; ++i){
      sumVec=bVal(i); 
      for(k=i-1; k>=r; k--){
        sumVec-=LVal(i,k)*xVal(k);
      }
      xVal(i)=sumVec/LVal(i,i);
    }
  
    dvar_matrix x=nograd_assign(xVal);
    save_identifier_string("TEST1");
    L.save_dvar_matrix_value();
    L.save_dvar_matrix_position();
    b.save_dvar_matrix_value();
    b.save_dvar_matrix_position();
    x.save_dvar_matrix_value();
    x.save_dvar_matrix_position();
    save_identifier_string("TEST6");
    gradient_structure::GRAD_STACK1->
             set_gradient_stack(dmdm_bksolve);
    RETURN_ARRAYS_DECREMENT();
    return x;
  }
  
  void dmdm_bksolve(void)
  {
    verify_identifier_string("TEST6");
    dvar_matrix_position xpos=restore_dvar_matrix_position();
    dmatrix dfx=restore_dvar_matrix_derivatives(xpos);
    dmatrix x=restore_dvar_matrix_value(xpos);
    dvar_matrix_position bpos=restore_dvar_matrix_position();
    dmatrix b=restore_dvar_matrix_value(bpos);
    dvar_matrix_position Lpos=restore_dvar_matrix_position();
    dmatrix L=restore_dvar_matrix_value(Lpos);
    verify_identifier_string("TEST1");
  
    int i, k, r, R, c, C;
    r=b.rowmin();  
    R=b.rowmax();  
    c=b.colmin();
    C=b.colmax();
  
    dmatrix dfL(Lpos);
    dmatrix dfb(bpos);
    dvector sumVec(c,C);
    dvector dfsumVec(c,C);
    dfL.initialize();
    dfb.initialize();
    dfsumVec.initialize();
  
  
    for(i=R; i>=r; --i){
      //bring back sumVec 
      sumVec=x(i)*L(i,i);
  
      //xVal(i)=sumVec/LVal(i,i);
      dfsumVec+=dfx(i)/L(i,i);
      dfL(i,i)+= -(dfx(i)*sumVec)/(L(i,i)*L(i,i)); 
      dfx(i)=0.0;
  
      for(k=r; k<=i-1; k++){
        //sumVec-=LVal(i,k)*xVal(k);
        dfL(i,k)-=dfsumVec*x(k);
        dfx(k)-=dfsumVec*L(i,k);
  
      }
      //sumVec=bVal(i); 
      dfb(i)+=dfsumVec; 
      dfsumVec=0.0;
    }
  
    dfL.save_dmatrix_derivatives(Lpos);
    dfb.save_dmatrix_derivatives(bpos);
  }
  
  dvar_vector nLogNormal(_CONST dvar_vector& x, _CONST dvar_matrix& mu, _CONST dvar_matrix& S)
  {
    RETURN_ARRAYS_INCREMENT();    
    int r, R, c, C, N;
    r=mu.rowmin();
    R=mu.rowmax();
    c=mu.colmin();
    C=mu.colmax();
    N=R-r+1;
    dvar_vector ret(c,C);
    dvar_matrix diff(r,R,c,C);
    for(int i=c; i<=C; ++i)diff.colfill(i,x-column(mu,i));
    dvariable logDet;
    dvar_matrix chol=choleski_decomp(S);
    logDet=2.0*sum(log(diagonal(chol)));
    dvar_matrix tmp=bksolve(chol,diff);
    ret=0.5*(log2pi*N+logDet+colsum(square(tmp)));
    RETURN_ARRAYS_DECREMENT();
    return ret;
  }

  dvariable nLogNormal(_CONST dvar_vector& x, _CONST dvar_vector& mu, _CONST dvar_matrix& S)
  {
    RETURN_ARRAYS_INCREMENT();    
    dvar_matrix MU(mu.indexmin(),mu.indexmax(),1,1);
    MU.colfill(1,mu);
    dvar_vector tmp=nLogNormal(x, MU, S);
    dvariable ret=tmp(tmp.indexmin());    
    RETURN_ARRAYS_DECREMENT();
    return ret;
  }

DATA_SECTION
  init_int noObs
  init_matrix obs(1,noObs,1,2)
  vector X0(1,2)  
  
PARAMETER_SECTION
  init_vector logK(1,3);
  init_number logSigma;
  init_number logTau(2);
  init_number logRho(2);

  sdreport_vector k(1,3);
  sdreport_number sigma2;
  sdreport_number tau2;
  sdreport_number rho2;
  sdreport_vector M(1,noObs);

  matrix X(1,noObs,1,2);
  matrix A(1,2,1,2);
  matrix S(1,noObs,1,noObs);
  //sdreport_vector Mres(1,noObs);
  objective_function_value nll;

PRELIMINARY_CALCS_SECTION
  X0(1)=0.0; X0(2)=100.0;
  logK=-2.0;
  logSigma=-2;
  logRho=2;
  logTau=-1;

PROCEDURE_SECTION
  k=exp(logK);
  sigma2=exp(2.0*logSigma);
  tau2=exp(2.0*logTau);
  rho2=exp(2.0*logRho);

  A(1,1)= -k(1); A(1,2)=  k(2);      
  A(2,1)=  k(1); A(2,2)= -k(2)-k(3); 

  S.initialize();
  for(int i=1; i<=noObs; ++i){
    for(int j=i; j<=noObs; ++j){
      S(i,j)=tau2*exp(-square(obs(i,1)-obs(j,1))/rho2);
      S(j,i)=S(i,j);
    }
    S(i,i)+=sigma2;
  }

  for(int i=1; i<=noObs; ++i){
    X(i)=expm(A*obs(i,1))*X0;
    M(i)=100.0-sum(X(i));
    //Mres(i)=obs(i,2)-M(i);
  }

  //nll=0.5*(log(2.0*M_PI)*noObs+log(det(S))+Mres*inv(S)*Mres);
  nll=nLogNormal(column(obs,2),M,S);

REPORT_SECTION
  for(int i=1; i<=noObs; ++i){
    report<<obs(i,1)<<" "<<obs(i,2)<<" "<<X(i,1)<<" "<<X(i,2)<<" "<<M(i)<<endl;
  }

