GLOBALS_SECTION
  #include "admodel.h"
  const double pi = 3.141592654;

DATA_SECTION
  init_int n
  init_number priorFlag
  init_number priorMu
  init_number priorSigma
  init_matrix data(1,n,1,2)
  vector S(1,n)
  !! S = column(data,1);
  vector R(1,n)
  !! R = column(data,2);

PARAMETER_SECTION
  init_number a
  // init_bounded_number a(0,1)
  init_number b
  init_number logSigma
  sdreport_number sigma
  vector Rfit(1,n)
  number RSS
  number L1
  number L2
  objective_function_value f

INITIALIZATION_SECTION
  a 0.1
  b 0.001
  logSigma -1.0

PROCEDURE_SECTION
  Rfit = elem_div(S, a+b*S);
  RSS = sum(square(log(R)-log(Rfit)));
  sigma = exp(logSigma);
  L1 = 0.5*n*log(2*pi) + n*logSigma + RSS/(2.0*square(sigma));
  if(priorFlag == 0)
    L2 = 0.0;
  else
    L2 = 0.5*log(2*pi) + log(priorSigma) + square(a-priorMu)/(2.0*square(priorSigma));
  f = L1 + L2;

REPORT_SECTION
  report << "# priorFlag priorMu priorSigma" << endl;
  report << priorFlag << " " << priorMu << " " << priorSigma << endl << endl;
  report << "# a b logSigma sigma" << endl;
  report << a << " " << b << " " << logSigma << " " << sigma << endl << endl;
  report << "# RSS L1 L2 f" << endl;
  report << RSS << " " << L1 << " " << L2 << " " << f <<endl;
