GLOBALS_SECTION
  #include "admodel.h"
  const double pi = 3.141592654;

DATA_SECTION
  init_int k
  init_int n
  init_vector x(1,n)
  init_matrix y(1,n,1,k)

PARAMETER_SECTION
  init_number mu0
  init_number b1
  init_number logSigmaB(3)
  init_number logSigma

  random_effects_vector b(1,k,2)

  sdreport_number sigma
  sdreport_number sigmaB
  vector b0(1,k)
  matrix yfit(1,n,1,k)
  number RSS

  objective_function_value f

PROCEDURE_SECTION
  sigma = mfexp(logSigma);
  sigmaB = mfexp(logSigmaB);
  b0 = mu0 + b*sigmaB;
  for(int i=1; i<=n; i++)
    for(int j=1; j<=k; j++)
      yfit(i,j) = b0(j) + b1*x(i);
  RSS = sum(square(y-yfit));
  f =  0.5*k*n*log(2.0*pi) + k*n*logSigma + RSS/(2.0*square(sigma));
  f += 0.5*k*log(2.0*pi)   + sum(square(b))/2.0;

REPORT_SECTION
  report << "# mu0"    << endl << mu0    << endl
         << "# b"      << endl << b      << endl
         << "# b0"     << endl << b0     << endl
         << "# b1"     << endl << b1     << endl
         << "# RSS"    << endl << RSS    << endl
         << "# sigmaB" << endl << sigmaB << endl
         << "# sigma"  << endl << sigma  << endl
         << "# f"      << endl << f      << endl << endl;
  report << "# x"      << endl << x      << endl << endl;
  report << "# y"      << endl << y      << endl << endl;
  report << "# yfit"   << endl << yfit   << endl << endl;
