## All trees pooled

fm0 <- nls(circumference~phi1/(1+exp(-(age-phi2)/phi3)), data=Orange, start=list(phi1=200,phi2=800,phi3=400))

plot(circumference~age, data=Orange, pch=16)
x <- unique(Orange$age)
lines(x, predict(fm0,data.frame(age=x)), lwd=3)

########################################################################################################################

## Tree 3 only

fm1 <- nls(circumference~phi1/(1+exp(-(age-phi2)/phi3)), data=Orange, subset=Tree==3, start=list(phi1=200,phi2=800,phi3=400))



coef(fm1)
#     phi1     phi2     phi3
# 158.8294 734.8421 400.9549

phi1: asymptote
phi2: inflection point
phi3: scale parameter for x

x <- seq(0, 1600, 10)

y = Linf / (1 + exp(-(x-x50)/scale))

yfit <- phi1 / (1 + exp(-(x-phi2)/phi3))

yfit <- 158.8294 / (1 + exp(-(x-734.8421)/400.9549))



summary(fm1)$coefficients[,"Std. Error"]

# From summary.nls:

n <- length(residuals(fm1))
p <- length(coef(fm1))
resvar <- deviance(fm1) / (n-p)
XtXinv <- chol2inv(fm1$m$Rmat())
se <- sqrt(diag(XtXinv) * resvar)

## 50% faster than
se <- summary(fm1)$coefficients[,"Std. Error"]

## 75% faster than
se <- sqrt(diag(vcov(fm1)))

########################################################################################################################

fm2 <- nlsList(circumference~phi1/(1+exp(-(age-phi2)/phi3))|Tree, data=Orange, start=list(phi1=200,phi2=800,phi3=400))

coef(fm2)
#       phi1     phi2     phi3
# 3 158.8294 734.8421 400.9549
# 1 154.1629 627.1923 362.5709
# 5 207.2579 861.3037 379.9465
# 2 218.9983 700.3450 332.4934
# 4 225.3038 710.7051 303.1438

sum(residuals(fm2)^2)
# 1273.595

########################################################################################################################

fm3old <- nlme(circumference~phi1/(1+exp(-(age-phi2)/phi3)), fixed=phi1+phi2+phi3~1, random=phi1~1|Tree, data=Orange, start=c(phi1=200,phi2=800,phi3=400))
## fm3new <- nlmer(circumference~SSlogis(age,phi1,phi2,phi3)~phi1|Tree, data=Orange, start=c(phi1=200,phi2=800,phi3=400))



coef(fm3old)
#       phi1     phi2     phi3
# 3 154.0446 722.5357 344.1529
# 1 161.6402 722.5357 344.1529
# 5 185.8674 722.5357 344.1529
# 2 222.6103 722.5357 344.1529
# 4 231.0652 722.5357 344.1529

## coef(fm3new)
#       phi1     phi2     phi3
# 3 154.1158 727.8909 347.9681
# 1 161.8970 727.8909 347.9681
# 5 186.7419 727.8909 347.9681
# 2 224.3931 727.8909 347.9681
# 4 233.0572 727.8909 347.9681



fixef(fm3old)
#     phi1     phi2     phi3
# 191.0455 722.5357 344.1529

## slot(fm3new, "fixef")
#     phi1     phi2     phi3
# 192.0411 727.8909 347.9681



ranef(fm3old)
#         phi1
# 3 -37.000971
# 1 -29.405325
# 5  -5.178094
# 2  31.564769
# 4  40.019621

## slot(fm3new, "ranef")
# -37.925308 -30.144042 -5.299149 32.351996 41.016109



fm3old
# StdDev: 31.48254 7.846255

## fm3new
# Groups   Name Variance  Std.Dev.
# Tree     phi1 53985.623 232.348
# Residual         52.868   7.271



sum(residuals(fm3old)^2)
# 1852.955

## sum(residuals(fm3new)^2)
# 1845.385



plot(augPred(fm3old,level=0:1), layout=c(1,5))



summary(fm3old)$tTable

# From summary.lme:
se     <- sqrt(diag(as.matrix(fm3old$varFix)))
se.adj <- sqrt(fm3old$dims$N/(fm3old$dims$N-length(se))) * se


x <- unique(Orange$age)
plot(circumference~age, data=Orange, col=as.character(Tree), pch=16)
lines(x, predict(fm3old,data.frame(age=x,Tree=1)), lwd=3, col=1)
lines(x, predict(fm3old,data.frame(age=x,Tree=2)), lwd=3, col=2)
lines(x, predict(fm3old,data.frame(age=x,Tree=3)), lwd=3, col=3)
lines(x, predict(fm3old,data.frame(age=x,Tree=4)), lwd=3, col=4)
lines(x, predict(fm3old,data.frame(age=x,Tree=5)), lwd=3, col=5)


########################################################################################################################

fm4 <- nlme(circumference~beta0+beta1*age, fixed=beta0+beta1~1, random=beta0~1|Tree, data=Orange, start=c(beta0=0,beta1=0))



plot(circumference~age, data=Orange, col=as.character(Tree), pch=16)
lines(age, predict(fm4,data.frame(age=age,Tree=1)), lwd=3, col=1)
lines(age, predict(fm4,data.frame(age=age,Tree=2)), lwd=3, col=2)
lines(age, predict(fm4,data.frame(age=age,Tree=3)), lwd=3, col=3)
lines(age, predict(fm4,data.frame(age=age,Tree=4)), lwd=3, col=4)
lines(age, predict(fm4,data.frame(age=age,Tree=5)), lwd=3, col=5)

########################################################################################################################

